unit ldn187u1;

//********************************************
(*
    LDN187 derived from GPC001. Work started
    14 Feb 17, inspired by ESP8266/ IfTTT

    This software goes with web tutorial...
    http://sheepdogguides.com/lut/lt5N.htm

    A simple demo of a program which can send
    URLs off to servers on internet or LAN.

    Intended for use with servers which are
    IoT accutators... i.e. something that
    "does something" when it recieves the sort
    of command this program can send... e.g.
    turn an LED attached to the server on or off.

    (Note to self... arises out of
       Hope of creating a new blinds closer...
       An Arduino would deal with blinds, a
       Windows PC progra would arrange for the
       right command to be sent at the right
       time.)
                                               *)
//---------------------------------------------
//*********************************************


(*==============================================
** MIGHT POINT TO IMPORTANT GENERAL ISSUES...
TI had to make the separator between IP-addr+port
and command \, not /. (With \ got "failed" when
trying to access ESP8266 webserver/ Arduserver.
Odd.

May just be to do with command the Arduserver
is expecting.

May not BE a separator... may just be part of
the command.
                                               *)
//---------------------------------------------
//*********************************************

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, httpsend, Inifiles;

type

  { TLDN187f1 }

  TLDN187f1 = class(TForm)
    buQuit: TButton;
    buDoIt: TButton;
    buSaveSettings: TButton;
    eUserNote0: TEdit;
    eLoadFromPageName: TEdit;
    eLoadFromIPAddr: TEdit;
    eUserNote1: TEdit;
    laTutURLTxt: TLabel;
    laSourceCommandHelp: TLabel;
    laSourceCommandHelp1: TLabel;
    laSrcPathTxt: TLabel;
    laSrcCmnd: TLabel;
    laHistory: TLabel;
    laSynapseURL: TLabel;
    meLog: TMemo;
    procedure buDoItClick(Sender: TObject);
    procedure buQuitClick(Sender: TObject);
    procedure buSaveSettingsClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    boTmp,boUserToldSettingsSavedOnExit:boolean;
    iTmp:integer;
    dfIniFile:TIniFile;
    sTmp,sSaveToFileSuffixPlusExtn:string;
    function boDownloadHTTP(URL, TargetFile: string): Boolean;
    procedure DoIt;
    procedure SaveStuffToIniFile(sWhere:string);
    procedure LoadStuffFromIniFile;
  end;

var
  LDN187f1: TLDN187f1;

const
  vers='25 Jan 16';
  //Created 14 Feb 17, by cutting down GPC001

implementation

{$R *.lfm}

{ TLDN187f1 }

procedure TLDN187f1.DoIt{(boRegardlessOfTime:boolean;bHrNow:byte)};
var URL:string;

begin
   //Build URL. This code doesn't catch every possible
   //  syntax error, but it makes a start.
   URL:=eLoadFromIPAddr.Text;
   URL:=URL+eLoadFromPageName.Text;

   {We don't actually save to the file, in the program
    as implemented for our very narrow wants... but the
    program makes a good basis for other work where what
    comes back from the webserver is saved, and it is
    easiest just to leave this in, here.}
   sSaveToFileSuffixPlusExtn:='tmp17214.txt';

   {The DownloadHTTP we are about to do will go off to
    a web (or LAN) address, bearing a message. The
    server may send something back. In the current program,
    with its narrow wants, nothing is done with what comes
    back... but it COULD have been saved, during DownloadHTTP,
    and to make it easy to adapt this program for other wants,
    the spec for WHERE to save what is returned is left in
    the structure of DownloadHTTP. See also its code.}

   if boDownloadHTTP(URL,sSaveToFileSuffixPlusExtn)//no ; here
     then sTmp:='Success!'{+sSaveToFileSuffixPlusExtn}// no ; here
     else sTmp:='Failed. Tried to access: '+URL;

   meLog.lines.insert(0,sTmp+' at '+DateTimeToStr(Now));
end;//DoIt

function TLDN187f1.boDownloadHTTP(URL, TargetFile:string): Boolean;
var
  HTTPGetResult: Boolean;
  HTTPSender: THTTPSend;
begin
  Result := False;
  HTTPSender := THTTPSend.Create;
  try
    HTTPGetResult := HTTPSender.HTTPMethod('GET', URL);
    if (HTTPSender.ResultCode >= 100) and (HTTPSender.ResultCode<=299) then begin
    {The following is not needed in our very specific use of HTTPSender,
       and remming it out saves disk thrash. I have only remmed it out
       rather than deleting it, in case someone wants to adapt this code,
       and needs something sent back, in circumstances other than
       just sending an instruction to an IoT "output" device.)
       3 of 3 rem outs for this.
      HTTPSender.Document.SaveToFile(TargetFile);
      }
      Result := True;
    end;
  finally
    HTTPSender.Free;
  end;
end; //DownloadHTTP

procedure TLDN187f1.FormCreate(Sender: TObject);
begin
 boUserToldSettingsSavedOnExit:=false;
 application.title:='LDN187';
 caption:='LDN187- First steps with TCP/IP- send a URL to web or LAN. Vers: '+vers;

 LoadStuffFromIniFile;

 meLog.lines.clear;
 meLog.lines.Add('Prgm start: '+DateTimeToStr(Now));
end; //FormCreate

procedure TLDN187f1.LoadStuffFromIniFile;
begin
dfIniFile:=TIniFile.Create(ExtractFilePath(Application.ExeName)+'LDN187ini.txt');

with dfIniFile do begin
 sTmp:=readstring('MainElements','IPAddr','bbc.co.uk');
 eLoadFromIPAddr.Text:=sTmp;
 sTmp:=readstring('MainElements','Page','index.htm');
 eLoadFromPageName.Text:=sTmp;

 LDN187f1.left:=ReadInteger('WindowPosnAndSize','Left',10);
 LDN187f1.top:=ReadInteger('WindowPosnAndSize','Top',10);
 LDN187f1.width:=ReadInteger('WindowPosnAndSize','Width',400);
 LDN187f1.height:=ReadInteger('WindowPosnAndSize','Height',300);

 sTmp:=readstring('Optional','UserNote0','Notes can be entered here. They are saved...');
 eUserNote0.Text:=sTmp;

 sTmp:=readstring('Optional','UserNote1','... in the ini file, reloaded next time.');
 eUserNote1.Text:=sTmp;

 end;//with...

dfIniFile.Free;
end;//LoadStuffFromIniFile

procedure TLDN187f1.SaveStuffToIniFile(sWhere:string);
begin
   dfIniFile:=TIniFile.Create(sWhere);

  with dfIniFile do begin
   WriteString('MainElements','IPAddr',LDN187f1.eLoadFromIPAddr.Text);
   WriteString('MainElements','Page',LDN187f1.eLoadFromPageName.Text);

   WriteInteger('WindowPosnAndSize','Left',LDN187f1.left);
   WriteInteger('WindowPosnAndSize','Top',LDN187f1.top);
   WriteInteger('WindowPosnAndSize','Width',LDN187f1.width);
   WriteInteger('WindowPosnAndSize','Height',LDN187f1.height);

 //  WriteInteger('Times','StartHr',LDN187f1.bHrStrt);
 //  WriteInteger('Times','LastHr',LDN187f1.bHrLast);

   WriteString('Optional','UserNote0',LDN187f1.eUserNote0.Text);
   WriteString('Optional','UserNote1',LDN187f1.eUSerNote1.Text);
   end;//with...

 dfIniFile.Free;

end;//SaveStuffTIni

procedure TLDN187f1.buSaveSettingsClick(Sender: TObject);
begin
  sTmp:=ExtractFilePath(Application.ExeName)+'LDN187ini.txt';
  SaveStuffToIniFile(sTmp);
  //N.B.... there is more than one call of "save stuff"... be sure
  //where you save it doesn't change!
  if NOT boUserToldSettingsSavedOnExit then begin
  showmessage('Settings are also saved on exit');
  boUserToldSettingsSavedOnExit:=true;
  end;
end;

procedure TLDN187f1.FormResize(Sender: TObject);
begin
  meLog.top:=162;
  meLog.height:=LDN187f1.height-280;
end;

procedure TLDN187f1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
 SaveStuffToIniFile(ExtractFilePath(Application.ExeName)+'LDN187ini.txt');//ALL refs to this file should be via one constant!
  //N.B.... there is more than one call of "save stuff"... be sure
  //where you save it doesn't change!
end;//FormClose

procedure TLDN187f1.buDoItClick(Sender: TObject);
begin
  DoIt;
end;

procedure TLDN187f1.buQuitClick(Sender: TObject);
begin
  close;
end;

end.

